#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "dsp_oddcastUI.h"

#include <ctype.h>

#include <wx/notebook.h>
#include <wx/image.h>
#include <wx/statline.h>
#include <wx/valtext.h>
#include <wx/valgen.h>

#include "log.h"
#include "transcurl.h"
#include "liboddcast.h"
#include "transcode.h"
#ifndef WIN32
#include <unistd.h>
#include <signal.h>
#endif
#include <math.h>

#ifdef WIN32
#define ODDCAST_BITMAP "oddcastv2.bmp"
#endif

#define VERSION_STRING __DATE__ " " __TIME__
#define foo_oddcast "foo_oddcast"
#define wa2_oddcast "wa2_oddcast"
#define xmms_oddcast "xmms_oddcast"


#define ID_CONNECT_BUTTON	100
#define ID_CONFIG_BUTTON	101

transcodeGlobals	globals;

// Declare two frames
MyFrame   *frame = (MyFrame *) NULL;
wxMenuBar *menu_bar = (wxMenuBar *) NULL;
FILE	*namedPipe = NULL;
static int gAvgLeft = 0;
static int gAvgRight = 0;

static int shown = 1;

#ifndef WIN32
void usr1handler (int sig)
{
  exit(1);
}
#endif

#ifndef WIN32
IMPLEMENT_APP(MyApp)
#else
IMPLEMENT_APP_NO_MAIN(MyApp)
#endif
DECLARE_APP(MyApp)

int	connectButtonPressed = 0;
int configButtonPressed = 0;
int vuButtonPressed = 0;
int mdButtonPressed = 0;
ReconnectTimer	reconnectTimer;
VUTimer	vuTimer;

MetatdataUpdateDlg	*metadataDialog;

void stopOddcast();
int  startOddcast();

void setUIMetadata(char *pMetadata)
{
	if (getLockedMetadataFlag()) {
		if (setCurrentSongTitle((char *)getLockedMetadata())) {
			if (globals.inputMetadataCallback) {
				globals.inputMetadataCallback((void *)getLockedMetadata());
			}
		}
	}
	else {
		if (setCurrentSongTitle((char *)pMetadata)) {
			if (globals.inputMetadataCallback) {
				globals.inputMetadataCallback((void *)pMetadata);
			}
		}
	}
}

int reconnectServer()
{
	stopOddcast();
	return startOddcast();
}

void startItUp() {
	MyApp	*pApp;

	pApp = (MyApp *)&wxGetApp();
}

int frameShowHide(bool which) {
	if (frame) {
		frame->Show(which);
		return 1;
	}
	else {
		return 0;
	}
}
void closeItDown() {
	MyApp	*pApp;

	pApp = &wxGetApp();
	/*
	if (pApp) {
		while (pApp->Pending()) {
			wxUsleep(300);
		}
	}
	*/
	if (pApp) {
//		frame->mainPanel->Close();
		pApp->OnExit();
		wxApp::CleanUp();
	}

}
void mainStatusCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();

	pApp->setMainStatus((char *)pValue);
}
void inputStatusCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();

	pApp->setInputStatus((char *)pValue);

}
void readBytesCallback(void *pValue) {
	// pValue is a long
	static	long	startTime = 0;	
	static	long	endTime = 0;	
	static	long	bytesReadInterval = 0;
	char	kBPSstr[25] = "";

	long	bytesRead = (long)pValue;

	MyApp	*pApp;

	pApp = &wxGetApp();

	if (bytesRead == -1) {
		strcpy(kBPSstr, "");
		pApp->setInputTotalRead((char *)kBPSstr);
		globals.totalBytesRead = 0;
		pApp->setInputBPSRead((char *)kBPSstr);
		startTime = 0;
		return;
	}

	if (startTime == 0) {
		startTime = time(&startTime);
		bytesReadInterval = 0;
	}
	bytesReadInterval += bytesRead;
	globals.totalBytesRead += bytesRead;
	endTime = time(&endTime);
	if ((endTime - startTime) > 5) {
		int bytespersec = bytesReadInterval/(endTime - startTime);
		long kBPS = (bytespersec * 8)/1000;
		sprintf(kBPSstr, "%ld Kbps", kBPS);
		pApp->setInputBPSRead((char *)kBPSstr);
		startTime = 0;
	}

	sprintf(kBPSstr, "%ld bytes", globals.totalBytesRead);
	pApp->setInputTotalRead((char *)kBPSstr);
}
void inputServerTypeCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setInputServerType((char *)pValue);
}
void inputStreamTypeCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setInputStreamType((char *)pValue);
}
void inputStreamURLCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setMainStreamURL((char *)pValue);
}
void inputStreamNameCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setMainStreamName((char *)pValue);
}
void inputMetadataCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setMainMetadata((char *)pValue);
	if (!frame->mainPanel->mouseDrag) {
		frame->mainPanel->Refresh();
	}
}
void inputBitrateCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setInputBitrate((char *)pValue);
}
void outputStatusCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setOutputStatus((char *)pValue);
	if (!frame->mainPanel->mouseDrag) {
		frame->mainPanel->Refresh();
	}
}
void writeBytesCallback(void *pValue) {
	// pValue is a long
	static	long	startTime = 0;	
	static	long	endTime = 0;	
	static	long	bytesWrittenInterval = 0;
	char	kBPSstr[25] = "";

	long	bytesWritten = (long)pValue;
	MyApp	*pApp;

	pApp = &wxGetApp();

	if (bytesWritten == -1) {
		strcpy(kBPSstr, "");
		pApp->setOutputTotalRead((char *)kBPSstr);
		globals.totalBytesWritten = 0;
		pApp->setOutputBPSRead((char *)kBPSstr);
		startTime = 0;
		return;
	}
	if (startTime == 0) {
		startTime = time(&startTime);
		bytesWrittenInterval = 0;
	}
	bytesWrittenInterval += bytesWritten;
	globals.totalBytesWritten += bytesWritten;
	endTime = time(&endTime);
	if ((endTime - startTime) > 4) {
		int bytespersec = bytesWrittenInterval/(endTime - startTime);
		long kBPS = (bytespersec * 8)/1000;
		sprintf(kBPSstr, "%ld Kbps", kBPS);
		pApp->setOutputBPSRead((char *)kBPSstr);
		startTime = 0;
		if (frame->mainPanel->kbpsSent_Rect) {
			if (!frame->mainPanel->mouseDrag) {
				//frame->mainPanel->RefreshRect(*frame->mainPanel->kbpsSent_Rect);
				frame->mainPanel->RefreshRect(*frame->mainPanel->kbpsSent_Rect);
			}
		}
	}

	//sprintf(kBPSstr, "%ld bytes", globals.totalBytesWritten);
	//pApp->setOutputTotalRead((char *)kBPSstr);
}
void outputServerTypeCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setOutputServerType((char *)pValue);
}
void outputStreamTypeCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setOutputStreamType((char *)pValue);
}
void outputServerNameCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setOutputServerName((char *)pValue);
}
void outputBitrateCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setOutputBitrate((char *)pValue);
}
void outputStreamURLCallback(void *pValue) {
	// pValue is a string
	MyApp	*pApp;

	pApp = &wxGetApp();
	pApp->setMainOutputURL((char *)pValue);
}

int gAvgSumLeft = 0;
int gAvgSumLeftCount = 0;
int gAvgSumRight = 0;
int gAvgSumRightCount = 0;

void VUCallback(int leftMax, int rightMax) {


	gAvgSumLeft += leftMax;
	gAvgSumRight += rightMax;
	gAvgSumLeftCount++;
	gAvgSumRightCount++;



}


int transcode_init(int argc, char **argv)
{
	char	*configFile = NULL;
	char	*defaultConfigFile = "transcoder.cfg";
	int	printConfig = 0;
	
	memset(&globals, '\000', sizeof(globals));

	// Setup Callbacks for all the interaction
	globals.mainStatusCallback = mainStatusCallback;
	globals.inputStatusCallback = inputStatusCallback;
	globals.inputStreamURLCallback = inputStreamURLCallback;
	globals.readBytesCallback = readBytesCallback;
	globals.inputServerTypeCallback = inputServerTypeCallback;
	globals.inputStreamTypeCallback = inputStreamTypeCallback;
	globals.inputStreamURLCallback = inputStreamURLCallback;
	globals.inputStreamNameCallback = inputStreamNameCallback;
	globals.inputMetadataCallback = inputMetadataCallback;
	globals.inputBitrateCallback = inputBitrateCallback;
	globals.outputStatusCallback = outputStatusCallback;
	globals.writeBytesCallback = writeBytesCallback;
	globals.outputServerTypeCallback = outputServerTypeCallback;
	globals.outputStreamTypeCallback = outputStreamTypeCallback;
	globals.outputServerNameCallback = outputServerNameCallback;
	globals.outputBitrateCallback = outputBitrateCallback;
	globals.outputStreamURLCallback = outputStreamURLCallback;

	setServerStatusCallback(globals.outputStatusCallback);
	setGeneralStatusCallback(globals.mainStatusCallback);
	setWriteBytesCallback(globals.writeBytesCallback);
	setBitrateCallback(globals.outputBitrateCallback);
	setStreamTypeCallback(globals.outputStreamTypeCallback);
	setServerNameCallback(globals.outputServerNameCallback);
	setServerTypeCallback(globals.outputServerTypeCallback);
	setSourceURLCallback(globals.inputStreamURLCallback);
	setDestURLCallback(globals.outputStreamURLCallback);
	setVUCallback(VUCallback);

	if (configFile == NULL) {
		configFile = defaultConfigFile;
	}
//	setConfigFileName(configFile);
	readConfigFile();
	if (printConfig) {
		printConfigFileValues();
	}

	setFrontEndType(FRONT_END_ODDCAST_PLUGIN);
	return 1;
}

void MyApp::setMainStatus(char *pValue)
{
	wxString	value((wxChar *)pValue);
}

void MyApp::setMainStreamURL(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setMainStreamName(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->serverDesc = value;
}
void MyApp::setMainMetadata(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->metadata = value;
}
void MyApp::setMainOutputURL(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->destination = value;
}
void MyApp::setInputStatus(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setInputTotalRead(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setInputBPSRead(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setInputServerType(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setInputStreamType(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setInputBitrate(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setOutputStatus(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->status = value;
}
void MyApp::setOutputTotalRead(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setOutputBPSRead(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->kbpsSent = value;
}
void MyApp::setOutputServerType(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setOutputStreamType(char *pValue)
{
	wxString	value((wxChar *)pValue);

}
void MyApp::setOutputServerName(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->serverDesc = value;
}
void MyApp::setOutputBitrate(char *pValue)
{
	wxString	value((wxChar *)pValue);

	frame->mainPanel->bitrate = value;
}
MyApp::MyApp()
{
}

int MyApp::OnExit() {
	if (frame) {
		delete frame;
	}
	reconnectTimer.Stop();
	vuTimer.Stop();
	while (reconnectTimer.IsRunning()) {
		::wxUsleep(300);
	}

	return wxApp::OnExit();
}
bool MyApp::OnInit()
{
  // Create the main frame window
	long basePanelHeight = 209;
	long basePanelWidth = 310;

	long	mainPanelWidth = basePanelWidth;
	long	mainPanelHeight = basePanelHeight;
	long	mainPanelX = 0;
	long	mainPanelY = 0;

	long initialWindowWidth = (mainPanelWidth);
	long initialWindowHeight = (mainPanelHeight);

#ifdef XMMS_PLUGIN
	char	programName[1024] = "";
	strcpy(programName, basename((char *)argv[0]));	
	if (argc < 4) {
		char buf[1024] = "";
		fprintf(stdout, "usage: %s /path/to/pipe logfile configfile\n", programName);
		exit(0);
	}

	char currentlogFile[1024] = "";
	char configFile[1024] = "";
	char pipeName[1024] = "";
	
	strcpy(pipeName, (char *)argv[1]);
	strcpy(currentlogFile, (char *)argv[2]);
	strcpy(configFile, (char *)argv[3]);

//	fprintf(stdout, "Current Directory (%s) log (%s) config(%s)\n", currentDir, currentlogFile, configFile);
//	fflush(stdout);
	setSourceURL(pipeName);
    setLogFile(currentlogFile);
    setConfigFileName(configFile);
	
	signal(SIGUSR1, usr1handler);
#endif

	readConfigFile();
	char	desc[255] = "";
	sprintf(desc, "Used for any window positions (X value)");
	int lastX = GetConfigVariableLong("","lastX",0,"", desc);
	sprintf(desc, "Used for any window positions (Y value)");
	int lastY = GetConfigVariableLong("","lastY",0,"", desc);

	frame = new MyFrame((MyFrame *) NULL, (char *) "Oddcast", (int)lastX, (int)lastY, initialWindowWidth, initialWindowHeight);

	frame->SetAutoLayout(TRUE);

  // Make a panel

	wxColor	colorRed(255,0,0);
	wxColor	colorGreen(0,255,0);
	wxColor	colorBlue(0,0,255);


	frame->mainPanel = new MainPanel(frame, mainPanelX, mainPanelY, mainPanelWidth, mainPanelHeight );
	frame->mainPanel->SetBackgroundColour(wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DFACE));

#ifdef WIN32
	if (m_nCmdShow == SW_SHOW) {
		frame->Show(TRUE);
	}
#else
		frame->Show(TRUE);
#endif


	initializeGlobals();
	transcode_init(0, NULL);
	setSourceURL(pipeName);

	if (getLockedMetadataFlag()) {
		setUIMetadata((char *)getLockedMetadata());
	}

	frame->SetSize((int)getLastXWindow(), (int)getLastYWindow(), -1, -1, wxSIZE_USE_EXISTING);

	frame->mainPanel->vu_onoff = getVUShow();

	SetTopWindow(frame);

	if (getReconnectFlag()) {
		reconnectTimer.Start(getReconnectSecs()*1000, FALSE);
	}
	vuTimer.Start(160, FALSE);
#ifdef WIN32

	wxInitAllImageHandlers();

	wxImage iconbm;

//	wxIconBundle	icon("oddcast.ico", wxBITMAP_TYPE_ICO);

//	icon.SetWidth(16);
//	icon.SetHeight(16);

//	iconbm.AddHandler(new wxICOHandler);
//	iconbm.LoadFile("oddcast.ico", wxBITMAP_TYPE_ICO);
	wxIcon	icon;
//	wxIcon  icon2;

	icon.LoadFile("oddcast.ico", wxBITMAP_TYPE_ICO);
//	icon2.LoadFile("oddcast.ico", wxBITMAP_TYPE_ICO);

//	wxIcon icon(wxT("oddcast"));

	frame->SetIcons(icon);

    if (m_systemtrayicon.SetIcon(icon, "Oddcast")) {
		frame->iconCreated = 1;
	}
	else {
		frame->iconCreated = 0;
	}

	

//	icon.CopyFromBitmap(iconbm.ConvertToBitmap());

#endif

#ifndef WIN32
	execThread = new SampleThread();

	if ( execThread->Create() != wxTHREAD_NO_ERROR ) {
		wxLogError((const wxChar *)"Can't create thread!");
	}
	else {
		execThread->Run();
	}
#endif
	char	versionString[256] = "";
	sprintf(versionString, "%s - %s", APP_NAME, VERSION_STRING);
	outputStatusCallback((char *)versionString);

	int x, y;
	frame->GetPosition(&x, &y);

	metadataDialog = new MetatdataUpdateDlg(frame,x,y);
	

	return TRUE;
}

//-----------------------------------------------------------------
//  MyFrame
//-----------------------------------------------------------------

// Define my frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
  wxFrame(frame, -1, (wxChar *)title, wxPoint(x, y), wxSize(w, h), wxSIMPLE_BORDER)
{
  mainPanel = (MainPanel *) NULL;
}

MyFrame::~MyFrame()
{
	if (mainPanel) {
		delete mainPanel;
	}
}

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(LAYOUT_EDIT_CONFIG, MyFrame::EditConfig)
  EVT_MENU(LAYOUT_QUIT, MyFrame::Quit)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(MainPanel, wxWindow)
EVT_PAINT(MainPanel::OnPaint)
EVT_LEAVE_WINDOW(MainPanel::OnLeftMouseButtonUp)
EVT_ENTER_WINDOW(MainPanel::OnLeftMouseButtonUp)
EVT_MOTION(MainPanel::OnMouseMotion)
EVT_LEFT_DOWN(MainPanel::OnLeftMouseButtonDown)
EVT_LEFT_UP(MainPanel::OnLeftMouseButtonUp)
END_EVENT_TABLE()


void MyFrame::EditConfig(wxCommandEvent& WXUNUSED(event) )
{
#ifdef WIN32
	wxExecute("notepad transcoder.cfg", TRUE);
	transcode_init(0, NULL);
#else
	wxExecute((const wxChar *)"vi transcoder.cfg", TRUE);
	transcode_init(0, NULL);
#endif

}
void MainPanel::OnLeftMouseButtonDown(wxMouseEvent& event) {

	int	currentX = 0;
	int currentY = 0;

	parentFrame->GetPosition(&currentX, &currentY);

	CaptureMouse();
	wxGetMousePosition(&deltaMouseX, &deltaMouseY);

	lastMouseX = event.GetX();
	lastMouseY = event.GetY();

	int buttonPressed = 0;
	//mouseDrag = 1;
	if ((lastMouseX > 46) && (lastMouseX < 46+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		// Connect Button
		connectButtonPressed = 1;
		buttonPressed = 1;
		mouseDrag = 0;
		Refresh();
	}
	if ((lastMouseX > 225) && (lastMouseX < 225+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		// Config Button
		configButtonPressed = 1;
		buttonPressed = 1;
		mouseDrag = 0;
		Refresh();

	}
	if ((lastMouseX > 135) && (lastMouseX < 135+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		// VU Button
		vuButtonPressed = 1;
		buttonPressed = 1;
		mouseDrag = 0;
		Refresh();
	}
	if ((lastMouseX > 10) && (lastMouseX < 22) && (lastMouseY > 120) && (lastMouseY < 132)) {
		// Metadata Button Pressed
		mdButtonPressed = 1;
		buttonPressed = 1;
		mouseDrag = 0;
		Refresh();
	}

	mouseDown = 1;
}
void MainPanel::OnLeftMouseButtonUp(wxMouseEvent& event) {

	lastMouseX = event.GetX();
	lastMouseY = event.GetY();

	connectButtonPressed = 0;
	configButtonPressed = 0;
	vuButtonPressed = 0;
	mdButtonPressed = 0;

//	char buf[255] = "";
//	sprintf(buf, "(%d,%d)", lastMouseX, lastMouseY);
//	wxMessageDialog	connectit(this, buf, "Message", wxOK);
//	connectit.ShowModal();
	if (mouseDown) {
		ReleaseMouse();
	}
	
	Refresh();

	mouseDown = 0;
	mouseDrag = 0;

	if ((lastMouseX > 285) && (lastMouseX < 303) && (lastMouseY > 3) && (lastMouseY < 15)) {
		// Minimize
#ifdef WIN32
		if (frame->iconCreated) {
			frame->Show(FALSE);
		}
		else {
			frame->Iconize(TRUE);
		}
#else
		frame->Iconize(TRUE);
#endif
		shown = 0;
		return;
	}
	if ((lastMouseX > 7) && (lastMouseX < 18) && (lastMouseY > 5) && (lastMouseY < 12)) {
		// Close Box
#ifndef ODDCAST_PLUGIN
		exit(0);
#endif
		return;
	}
	if ((lastMouseX > 46) && (lastMouseX < 46+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		// Connect Button
		Connect();
	}
	if ((lastMouseX > 225) && (lastMouseX < 225+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		// Connect Button
		Config();
	}
	if ((lastMouseX > 135) && (lastMouseX < 135+50) && (lastMouseY > 159) && (lastMouseY < 159+20)) {
		VUOnOff();
	}
	if ((lastMouseX > 10) && (lastMouseX < 22) && (lastMouseY > 120) && (lastMouseY < 132)) {
		// Metadata Button Pressed
		if (!metadataDialog->IsShown()) {
			metadataDialog->m_metadataLocked = getLockedMetadataFlag();
			if (metadataDialog->m_metadataLocked) {
				metadataDialog->m_Metadata = (wxChar *)getLockedMetadata();
			}
			else {
				metadataDialog->m_Metadata = (wxChar *)getCurrentlyPlaying();
			}
			if (metadataDialog->ShowModal() == wxID_OK) {
				setLockedMetadata((char *)metadataDialog->m_Metadata.c_str());
				setLockedMetadataFlag(metadataDialog->m_metadataLocked);
				setUIMetadata((char *)metadataDialog->m_Metadata.c_str());
				writeConfigFile();
			}
		}
	}
}
void MainPanel::OnMouseMotion(wxMouseEvent& event) {
	int	currentX = 0;
	int currentY = 0;
	int	deltaX = 0;
	int deltaY = 0;
	int currentMouseX = 0;
	int currentMouseY = 0;

	if (event.Moving()) {
		if (mouseDown) {
//			mouseDrag = 1;
			parentFrame->GetPosition(&currentX, &currentY);
			wxGetMousePosition(&currentMouseX, &currentMouseY);

			deltaX = currentMouseX - deltaMouseX;
			deltaY = currentMouseY - deltaMouseY;

			//fprintf(stdout, "current x,y = (%d,%d)\n", currentX, currentY);

			currentX = currentX + deltaX;
			currentY = currentY + deltaY;

			if ((getLastXWindow() != currentX) && (getLastYWindow() != currentY)) {
			//fprintf(stdout, "new     x,y = (%d,%d)\n\n", currentX, currentY);

				parentFrame->Move(currentX, currentY);

				deltaMouseX = currentMouseX;
				deltaMouseY = currentMouseY;

				setLastXWindow(currentX);
				setLastYWindow(currentY);
				mouseDrag = 1;
			}
		}
	}
}
void MyFrame::Quit(wxCommandEvent& WXUNUSED(event) )
{
    this->Close(TRUE);
}


//-----------------------------------------------------------------
//  MyWindow
//-----------------------------------------------------------------

BEGIN_EVENT_TABLE(MyWindow, wxWindow)
    EVT_PAINT(MyWindow::OnPaint)
END_EVENT_TABLE()

// Define a constructor for my canvas
MyWindow::MyWindow(wxFrame *frame, int x, int y, int w, int h, long style):
 wxWindow(frame, -1, wxPoint(x, y), wxSize(w, h), style)
{
}

MyWindow::~MyWindow()
{
}

// Define the repainting behaviour
void MyWindow::OnPaint(wxPaintEvent& WXUNUSED(event) )
{
  wxPaintDC dc(this);
  
}
void MainPanel::OnPaint(wxPaintEvent& WXUNUSED(event) )
{
	wxColour	fontColor;
	wxPen		fontPen;
	wxBrush		fontBrush;
	wxFont		font;

//	wxPaintDC Windc(this);
	wxPaintDC dc(this);
	int	windowWidth = 350;
	int	windowHeight = 0;


	if (mouseDrag) {
		return;
	}


	if (pBaseBitmap) {
		wxMemoryDC memDC;
		memDC.SelectObject(*pBaseBitmap);
		windowHeight = pBaseBitmap->GetHeight();
		dc.Blit(0, 0, 350, pBaseBitmap->GetHeight(), &memDC, 0, 0);
		memDC.GetPixel(360, 10, &fontColor);
	}

	fontPen.SetColour(fontColor);
	fontBrush.SetColour(fontColor);
	dc.SetPen(fontPen);
	dc.SetBrush(fontBrush);
	dc.SetTextForeground(fontColor);
	dc.SetBackgroundMode(wxTRANSPARENT);


#ifdef WIN32
	font.SetPointSize(8);
#else
	font.SetPointSize(10);
#endif

	dc.SetFont(font);

	long textRowSize = 20;
	long initialX = 10;
	long initialY = 25;
	int i = 0;



	dc.DrawText(serverDesc_label, initialX, initialY+(textRowSize*i));

	i++;
	dc.DrawText(destination_label, initialX, initialY+(textRowSize*i));
	i++;
	dc.DrawText(kbpsSent_label, initialX, initialY+(textRowSize*i));
	i++;
	dc.DrawText(bitrate_label, initialX, initialY+(textRowSize*i));
	i++;
	dc.DrawText(metadata_label, initialX, initialY+(textRowSize*i));
	i++;


	initialX = 75;
	i = 0;
	dc.DrawText(serverDesc, initialX, initialY+(textRowSize*i));
	if (!serverDesc_Rect) {
		serverDesc_Rect = new wxRect(initialX, initialY+(textRowSize*i), pBaseBitmap->GetWidth()-initialX, font.GetPointSize()+5);
	}
	i++;
	dc.DrawText(destination, initialX, initialY+(textRowSize*i));
	if (!destination_Rect) {
		destination_Rect = new wxRect(initialX, initialY+(textRowSize*i), pBaseBitmap->GetWidth()-initialX, font.GetPointSize()+5);
	}
	i++;
	dc.DrawText(kbpsSent, initialX, initialY+(textRowSize*i));
	if (!kbpsSent_Rect) {
		kbpsSent_Rect = new wxRect(initialX, initialY+(textRowSize*i), pBaseBitmap->GetWidth()-initialX, font.GetPointSize()+5);
	}
	i++;
	dc.DrawText(bitrate, initialX, initialY+(textRowSize*i));
	if (!bitrate_Rect) {
		bitrate_Rect = new wxRect(initialX, initialY+(textRowSize*i), pBaseBitmap->GetWidth()-initialX, font.GetPointSize()+5);
	}
	i++;
	char	formattedMetadata[50];
	memset(formattedMetadata, '\000', sizeof(formattedMetadata));
	strncpy(formattedMetadata, metadata, 44);
	if (strlen(metadata) > 44) {
		strcat(formattedMetadata, "...");
	}
	dc.DrawText(formattedMetadata, initialX, initialY+(textRowSize*i));
	if (!metadata_Rect) {
		metadata_Rect = new wxRect(initialX, initialY+(textRowSize*i), pBaseBitmap->GetWidth()-initialX, font.GetPointSize()+5);
	}
	i++;

//	char	sstatus[1024] = "";
//	sprintf(sstatus, "Left (%d) Right (%d)", gAvgLeft, gAvgRight);
//	status = sstatus;
	dc.DrawText(status, 5, 209-textRowSize+2);
	i++;


	if (vu_onoff) {
		int h = 15;
		int w = 120;
		int x = 100;
		int y = 130;
		


		int leftVUdata = 0;
		if (gAvgSumLeftCount > 0) {
			leftVUdata = gAvgSumLeft/gAvgSumLeftCount;
		}
		int rightVUdata = 0;
		if (gAvgSumRightCount > 0) {
			rightVUdata = gAvgSumRight/gAvgSumRightCount;
		}

		gAvgSumLeft = 0;
		gAvgSumLeftCount = 0;
		gAvgSumRight = 0;
		gAvgSumRightCount = 0;



		/*
		if (gAvgLeft > 0) {
			leftVUdata = (int)(gAvgLeft/800);
		}
		if (gAvgRight > 0) {
			rightVUdata = (int)(gAvgRight/800);
		}
		*/
		//
		//  Then, after we get the VU data, we make a graphical representation of it.
		//

		
		int vu_width = w-10;

	//
	//  *** Left VU
	//

		wxColor greenColor(0,255,0);
		wxColor darkgreenColor(0,90,0);
		wxColor redColor(255,0,0);
		wxColor darkredColor(90,0,0);
		wxColor yellowColor(255,255,0);
		wxColor darkyellowColor(90,90,0);
		wxColor blackColor(0,0,0);
		wxBrush greenBrush(greenColor, wxSOLID);
		wxBrush darkgreenBrush(darkgreenColor, wxSOLID);
		wxBrush redBrush(redColor, wxSOLID);
		wxBrush darkredBrush(darkredColor, wxSOLID);
		wxBrush yellowBrush(yellowColor, wxSOLID);
		wxBrush blackBrush(blackColor, wxSOLID);
		wxBrush darkyellowBrush(darkyellowColor, wxSOLID);
		wxPen greenPen(greenColor, 1, wxSOLID);
		wxPen darkgreenPen(darkgreenColor, 1, wxSOLID);
		wxPen redPen(redColor, 1, wxSOLID);
		wxPen darkredPen(darkredColor, 1, wxSOLID);
		wxPen yellowPen(yellowColor, 1, wxSOLID);
		wxPen darkyellowPen(darkyellowColor, 1, wxSOLID);
		wxPen blackPen(blackColor, 2, wxSOLID);
		int leftvux = x;
		int leftvuy = y;
		int leftvuh = (h/2)-1;
		int leftvuw = 3;
		int leftvuspacing = 0;

		double logData = 0.0;
		if (leftVUdata > 0) {
			logData = leftVUdata*1.5;
		}
		float pctlevel = ((float)leftVUdata/(float)15000.0);
		//float pctlevel = ((float)logData/(float)12.0);
		float leftvuwidth = (float)pctlevel * (float)w;
		
		float range1 = .59;
		float range2 = .79;
		float range3 = 1.00;
		float	rectanglewidth = 0;

		if (pctlevel >= 0) {
			dc.SetBrush(greenBrush);
			dc.SetPen(greenPen);
			if (pctlevel > range1) {
				rectanglewidth = range1*w + .5;
				dc.DrawRectangle(x, y, (int)rectanglewidth, leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w + .5;
				dc.DrawRectangle(x, y, (int)rectanglewidth, leftvuh);
				dc.SetBrush(darkgreenBrush);
				dc.SetPen(darkgreenPen);
				//dc.DrawRectangle(x+(int)rectanglewidth, y, (int)range1*w - rectanglewidth, leftvuh);
				int r2width = (range1*w + .5) - rectanglewidth + 1;
				dc.DrawRectangle(x+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		if (pctlevel >= range1) {
			dc.SetBrush(yellowBrush);
			dc.SetPen(yellowPen);
			if (pctlevel >= range2) {
				rectanglewidth = range2*w + .5 + 2;
				dc.DrawRectangle(x-1+(range1*w + .5), y, (int)rectanglewidth-(range1*w + .5), leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w + .5;
				dc.DrawRectangle(x-1+(range1*w + .5), y, (int)rectanglewidth-(range1*w + .5), leftvuh);
				dc.SetBrush(darkyellowBrush);
				dc.SetPen(darkyellowPen);
				int r2width = (range2*w + .5) - rectanglewidth + 2;
				dc.DrawRectangle(x-1+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		else {
			dc.SetBrush(darkyellowBrush);
			dc.SetPen(darkyellowPen);
			int r2width = (range2*w + .5) - (range1*w + .5) + 2;
			dc.DrawRectangle(x+(int)(range1*w + .5), y, (int)r2width, leftvuh);
		}
		if (pctlevel >= range2) {
			dc.SetBrush(redBrush);
			dc.SetPen(redPen);
			if (pctlevel >= range3) {
				rectanglewidth = range3*w+.5;
				dc.DrawRectangle(x+(range2*w + .5), y, (int)rectanglewidth-(range2*w + .5), leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w+.5;
				dc.DrawRectangle(x+(range2*w + .5), y, (int)rectanglewidth-(range2*w + .5), leftvuh);
				dc.SetBrush(darkredBrush);
				dc.SetPen(darkredPen);
				int r2width = range3*w - rectanglewidth + 2;
				dc.DrawRectangle(x+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		else {
			dc.SetBrush(darkredBrush);
			dc.SetPen(darkredPen);
			int r2width = (range3*w + .5) - (range2*w + .5) +1;
			dc.DrawRectangle(x+(int)(range2*w + .5), y, (int)r2width, leftvuh);
		}

		int rightvux = x;
		int rightvuy = y+(h/2)+1;
		int rightvuh = (h/2)-1;

		x = rightvux;
		y = rightvuy;

		int rightvuw = 3;
		int rightvuspacing = 0;

		logData = 0.0;
		if (rightVUdata > 0) {
			logData = rightVUdata*1.0;
		}
		pctlevel = ((float)logData/(float)15000.0);
		float rightvuwidth = (float)pctlevel * (float)w;
		
		rectanglewidth = 0;

		if (pctlevel >= 0) {
			dc.SetBrush(greenBrush);
			dc.SetPen(greenPen);
			if (pctlevel > range1) {
				rectanglewidth = range1*w + .5;
				dc.DrawRectangle(x, y, (int)rectanglewidth, leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w + .5;
				dc.DrawRectangle(x, y, (int)rectanglewidth, leftvuh);
				dc.SetBrush(darkgreenBrush);
				dc.SetPen(darkgreenPen);
				//dc.DrawRectangle(x+(int)rectanglewidth, y, (int)range1*w - rectanglewidth, leftvuh);
				int r2width = (range1*w + .5) - rectanglewidth + 1;
				dc.DrawRectangle(x+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		if (pctlevel >= range1) {
			dc.SetBrush(yellowBrush);
			dc.SetPen(yellowPen);
			if (pctlevel >= range2) {
				rectanglewidth = range2*w + .5 + 2;
				dc.DrawRectangle(x-1+(range1*w + .5), y, (int)rectanglewidth-(range1*w + .5), leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w + .5;
				dc.DrawRectangle(x-1+(range1*w + .5), y, (int)rectanglewidth-(range1*w + .5), leftvuh);
				dc.SetBrush(darkyellowBrush);
				dc.SetPen(darkyellowPen);
				int r2width = (range2*w + .5) - rectanglewidth + 2;
				dc.DrawRectangle(x-1+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		else {
			dc.SetBrush(darkyellowBrush);
			dc.SetPen(darkyellowPen);
			int r2width = (range2*w + .5) - (range1*w + .5) + 2;
			dc.DrawRectangle(x+(int)(range1*w + .5), y, (int)r2width, leftvuh);
		}
		if (pctlevel >= range2) {
			dc.SetBrush(redBrush);
			dc.SetPen(redPen);
			if (pctlevel >= range3) {
				rectanglewidth = range3*w+.5;
				dc.DrawRectangle(x+(range2*w + .5), y, (int)rectanglewidth-(range2*w + .5), leftvuh);
			}
			else {
				rectanglewidth = pctlevel*w+.5;
				dc.DrawRectangle(x+(range2*w + .5), y, (int)rectanglewidth-(range2*w + .5), leftvuh);
				dc.SetBrush(darkredBrush);
				dc.SetPen(darkredPen);
				int r2width = range3*w - rectanglewidth + 2;
				dc.DrawRectangle(x+(int)rectanglewidth, y, (int)r2width, leftvuh);

			}
		}
		else {
			dc.SetBrush(darkredBrush);
			dc.SetPen(darkredPen);
			int r2width = (range3*w + .5) - (range2*w + .5) +1;
			dc.DrawRectangle(x+(int)(range2*w + .5), y, (int)r2width, leftvuh);
		}
	}

	wxMemoryDC memDC;

	if (getIsConnected()) {
		memDC.SelectObject(pConnectGreenBitmap);
		dc.Blit(46, 159, pConnectGreenBitmap.GetWidth(), pConnectGreenBitmap.GetHeight(), &memDC, 0, 0);
	}
	else {
		memDC.SelectObject(pConnectRedBitmap);
		dc.Blit(46, 159, pConnectRedBitmap.GetWidth(), pConnectRedBitmap.GetHeight(), &memDC, 0, 0);
	}
	if (connectButtonPressed) {
		memDC.SelectObject(pConnectPressedBitmap);
		dc.Blit(46, 159, pConnectPressedBitmap.GetWidth(), pConnectPressedBitmap.GetHeight(), &memDC, 0, 0);
	}

	memDC.SelectObject(pConfigBitmap);
	dc.Blit(225, 159, pConfigBitmap.GetWidth(), pConfigBitmap.GetHeight(), &memDC, 0, 0);

	memDC.SelectObject(pVUBitmap);
	dc.Blit(135, 159, pVUBitmap.GetWidth(), pVUBitmap.GetHeight(), &memDC, 0, 0);

	memDC.SelectObject(pMetadataBitmap);
	dc.Blit(10, 120, pMetadataBitmap.GetWidth(), pMetadataBitmap.GetHeight(), &memDC, 0, 0);

	if (configButtonPressed) {
		memDC.SelectObject(pConfigPressedBitmap);
		dc.Blit(225, 159, pConfigPressedBitmap.GetWidth(), pConfigPressedBitmap.GetHeight(), &memDC, 0, 0);
	}
	if (vuButtonPressed) {
		memDC.SelectObject(pVUPressedBitmap);
		dc.Blit(135, 159, pVUPressedBitmap.GetWidth(), pVUPressedBitmap.GetHeight(), &memDC, 0, 0);
	}
	if (mdButtonPressed) {
		memDC.SelectObject(pMetadataPressedBitmap);
		dc.Blit(10, 120, pMetadataPressedBitmap.GetWidth(), pMetadataPressedBitmap.GetHeight(), &memDC, 0, 0);
	}
  
//	Windc.Blit(0, 0, windowWidth, windowHeight, &dc, 0, 0);
}

MainPanel::MainPanel( wxFrame *frame, int x, int y, int w, int h )
       : wxWindow( frame, -1, wxPoint(x, y), wxSize(w, h) )
{

	mouseDown = 0;
	mouseDrag = 0;
	parentFrame = frame;

	pBaseBitmap = new wxBitmap();
	if (!pBaseBitmap->LoadFile((const wxChar *)ODDCAST_BITMAP, wxBITMAP_TYPE_BMP)) {
		char msg[255] = "";
		sprintf(msg, "cannot open %s", ODDCAST_BITMAP);
		wxMessageDialog	connectit(this, (wxChar *)msg, (const wxChar *)"Message", wxOK);
		connectit.ShowModal();
	}
	else {
		int column = 0;
		wxRect	connectGreenRect(310+(50*column), 209-(20*2), 50, 20);
		pConnectGreenBitmap = pBaseBitmap->GetSubBitmap(connectGreenRect);
		wxRect	connectRedRect(310+(50*column), 209-(20*1), 50, 20);
		pConnectRedBitmap = pBaseBitmap->GetSubBitmap(connectRedRect);
		wxRect	connectPressedRect(310+(50*column), 209-(20*3), 50, 20);
		pConnectPressedBitmap = pBaseBitmap->GetSubBitmap(connectPressedRect);

		column = 1;
		wxRect	configRect(310+(50*column), 209-20, 50, 20);
		pConfigBitmap = pBaseBitmap->GetSubBitmap(configRect);
		wxRect	configPressedRect(310+(50*column), 209-(20*3), 50, 20);
		pConfigPressedBitmap = pBaseBitmap->GetSubBitmap(configPressedRect);

		column = 2;
		wxRect	VURect(310+(50*column), 209-20, 50, 20);
		pVUBitmap = pBaseBitmap->GetSubBitmap(VURect);
		wxRect	VUPressedRect(310+(50*column), 209-(20*3), 50, 20);
		pVUPressedBitmap = pBaseBitmap->GetSubBitmap(VUPressedRect);

		column = 3;
		wxRect	MDRect(310+(50*column), 209-12, 13, 12);
		pMetadataBitmap = pBaseBitmap->GetSubBitmap(MDRect);
		wxRect	MDPressedRect(310+(50*column), 209-(12*2), 13, 12);
		pMetadataPressedBitmap = pBaseBitmap->GetSubBitmap(MDPressedRect);

	}


	long textRowSize = 20;
	long initialX = 10;
	long initialY = 25;

	serverDesc_label = (const wxChar *)"Server Desc";
	destination_label = (const wxChar *)"Destination";
	metadata_label = (const wxChar *)"Metadata";
	kbpsSent_label = (const wxChar *)"kbps Sent";
	bitrate_label = (const wxChar *)"Bitrate";
	status_label = (const wxChar *)"Status";

	initialX = 275;
	int i = 2;

	serverDesc_Rect = NULL;
	destination_Rect = NULL;
	kbpsSent_Rect = NULL;
	bitrate_Rect = NULL;
	metadata_Rect = NULL;


	execThread = NULL;
	startThread = NULL;

}

void MainPanel::VUOnOff()
{
	if (vu_onoff == 1) {
		vu_onoff = 0;
	}
	else {
		vu_onoff = 1;
	}
	setVUShow(vu_onoff);
	Refresh();
}
void MainPanel::Config()
{
	char	commandToExecute[1024] = "";
	if (getIsConnected()) {
		wxMessageDialog	connectit(this, (const wxChar *)"Cannot edit settings while connected", (const wxChar *)"Message", wxOK);
		connectit.ShowModal();
	}
	else {

#ifdef WIN32
		memset(commandToExecute, '\000', sizeof(commandToExecute));
		sprintf(commandToExecute, "oddcast_editConfig.exe \"%s\"", getConfigFileName());
		int ret = wxExecute((wxChar *)commandToExecute, wxEXEC_SYNC, NULL);
		if (ret == -1) {
			wxMessageDialog	connectit(NULL, (const wxChar *)"Cannot seem to find oddcast_editConfig.exe, please re-install oddcast.", (const wxChar *)"Message", wxOK);
			connectit.ShowModal();
		}
#else
		memset(commandToExecute, '\000', sizeof(commandToExecute));
		sprintf(commandToExecute, "oddcast_config \"%s\"", getConfigFileName());
		int ret = wxExecute((wxChar *)commandToExecute, wxEXEC_SYNC, NULL);
		if (ret == 255) {
			wxMessageDialog	connectit(NULL, (const wxChar *)"Cannot seem to find oddcast_config,\nwhich means you either have not installed the \noddcast_config package or somehow it is not in your path.\nYou have two choices, you can get the oddcast_config package, \nor edit the config file manually.",(const wxChar *) "Message", wxOK);
			connectit.ShowModal();
		}
#endif
		else {
				initializeGlobals();
				transcode_init(0, NULL);
				if (reconnectTimer.IsRunning()) {
					reconnectTimer.Stop();
				}
				if (getReconnectFlag()) {
					reconnectTimer.Start(getReconnectSecs()*1000, FALSE);
				}

		}
	}
}

// Called when song changes
void UpdateWhatsPlaying()
{
	char	buf[4097] = "";
	sprintf(buf, "What's Playing");

}
void stopOddcast()
{
	disconnectFromServer();
	setForceStop(1);
	if (getLiveRecordingSetFlag()) {
		stopRecording();
	}
//	setUIMetadata("");

}
int startOddcast() 
{
	setForceStop(0);
	if (connectToServer()) {
		UpdateWhatsPlaying();
	}
	else {
		return(0);
	}
	if (getLiveRecordingSetFlag()) {
		if (!initLiveRecording()) {
			wxMessageDialog	connectit(NULL,(const wxChar *) "Fail to open input device",(const wxChar *) "Message", wxOK);
			connectit.ShowModal();
			disconnectFromServer();
			return 0;
		}
		else {
			char	buf[255] = "";
			sprintf(buf, "%dHz/Stereo PCM", getLiveInSamplerate());
			setSourceDescription(buf);
			setUIMetadata(getServerDesc());
			startRecording();
		}
	}
	return 1;
}

void MainPanel::Connect()
{
		if (getIsConnected()) {
			stopOddcast();			
		}
		else {
			char	message[1024] = "";
			char	error[1024] = "";
			int ret = 1;
#ifdef WIN32
			if (getWMAFlag()) {
				ret = startWMAServer(message,error);
			}
#endif
			if (ret == 0) {
				wxMessageDialog	connectit(this,(const wxChar *) message,(const wxChar *) "Message", wxOK);
				connectit.ShowModal();
				wxMessageDialog	connectit2(this,(const wxChar *) error,(const wxChar *) "Message", wxOK);
				connectit2.ShowModal();
			}
			else {
				startThread = new StartThread();

				if ( startThread->Create() != wxTHREAD_NO_ERROR ) {
					wxLogError("Can't create thread!");
				}
				else {
					startThread->Run();
				}

//				if (startOddcast()) {
//				}
			}
		}

#if 0
	if (execThread) {
		setStopTranscoder(1);
		// wxSleep(1);
		execThread->Delete();

		execThread = NULL;

		globals.mainStatusCallback((char *)"Stopped");
		globals.inputStatusCallback((char *)"Disconnected");
		globals.readBytesCallback((void *)-1);
		globals.inputServerTypeCallback((char *)"");
		globals.inputStreamTypeCallback((char *)"");
		globals.inputStreamNameCallback((char *)"");
		globals.inputMetadataCallback((char *)"");
		globals.inputBitrateCallback((char *)"");
		globals.writeBytesCallback((void *)-1);

		connectButton->SetBitmapLabel(pConnectRedBitmap);
	}
	else {
		setStopTranscoder(0);
		reset_transcoder();

		char	message[1024] = "";
		char	error[1024] = "";

		int ret = 1;


#ifdef WIN32
		if (getWMAFlag()) {
			ret = startWMAServer(message,error);
		}
#endif
		if (ret == 0) {
			outputStatusCallback((void *)message);
			globals.mainStatusCallback(error);

		}
		else {

			
			execThread = new MyThread();

			if ( execThread->Create() != wxTHREAD_NO_ERROR ) {
				wxLogError("Can't create thread!");
			}
			else {
				connectButton->SetBitmapLabel(pConnectGreenBitmap);

				execThread->Run();
			}
		}
		
	}
#endif
}
MainPanel::~MainPanel()
{
	if (pBaseBitmap) {
		delete pBaseBitmap;
	}
	if (execThread) {
		if (execThread->IsRunning()) {
			execThread->Delete();
			execThread->Wait();
		}
		delete execThread;
	}
	if (serverDesc_Rect) {
		delete serverDesc_Rect;
	}
	if (destination_Rect) {
		delete destination_Rect;
	}
	if (kbpsSent_Rect) {
		delete kbpsSent_Rect;
	}
	if (bitrate_Rect) {
		delete bitrate_Rect;
	}
	if (metadata_Rect) {
		delete metadata_Rect;
	}

}

StartThread::StartThread()
        : wxThread()
{
    m_count = 0;
}

void StartThread::OnExit()
{
	;
}

void *StartThread::Entry()
{
	startOddcast();
    return NULL;
}
MyThread::MyThread()
        : wxThread()
{
    m_count = 0;
}

void MyThread::OnExit()
{
	;
}

void *MyThread::Entry()
{
	char	buffer[255] = "";
	
	while (getIsConnected()) {
		fread(buffer, 1, 1, stdin);
		readBytesCallback((void *)1);
		//fprintf(stdout, "Read byte\n");
	}

//	wxMessageDialog	connectit2(NULL, "End Thread!!!", "Message", wxOK);
//	connectit2.ShowModal();
	//this->Exit((void *)1);
    return NULL;
}
SampleThread::SampleThread()
        : wxThread()
{
	;
}

void SampleThread::OnExit()
{
	;
}

#define BUFFER_LEN              (4096)

typedef struct OddAudiotag {
	int	nch;
	int	samplerate;
	int	numsamples;
	int	metadatalength;
	int	samplelength;
} OddAudio;

void *SampleThread::Entry()
{
#ifndef WIN32
	float *buffer;

	char	buf[255] = "";
	int	readcount;
	int	subformat;
	OddAudio	audioData;
	char	metadata[1024] = "";
	static	int startedReading = 0;
	fd_set  fdRead;
	fd_set  fdExcep;
	struct  timeval selectTimeout;

	selectTimeout.tv_sec = 0;
	selectTimeout.tv_usec = 500;
	int	fdSelected = 0;



	int ok = 1;
	struct stat statbuf;

		if (stat(getSourceURL(), &statbuf) == 0) {
			;
		}
		else {
			if (mkfifo(getSourceURL(), 0666) == 0) {
				//chmod(getSourceURL(), S_IRUSR | S_IWUSR | S_IXUSR);
				;
			}
			else {
				sprintf(buf, "Unable to create FIFO for %s", getSourceURL());
				ok = 0;
			}
		}
		namedPipe = fopen(getSourceURL(), "r");
		if (namedPipe == 0) {
			sprintf(buf, "Error reading from pipe");
			ok = 0;
		}
		//fcntl(fileno(namedPipe),F_SETFL,O_NDELAY);

		if (ok) {
			int loop = 1;
			FD_ZERO(&fdRead);
			FD_SET(fileno(namedPipe), &fdRead);
			while (loop) {
				//fdSelected = select(FD_SETSIZE, &fdRead, 0, 0, &selectTimeout);
				//if (FD_ISSET(fileno(namedPipe), &fdRead)) {
						memset(&audioData, '\000', sizeof(audioData));
						int read = fread(&audioData, 1, sizeof(audioData), namedPipe);
						if (read != sizeof(audioData)) {
							loop = 0;
							sprintf(buf, "Something terminated the pipe");
							fprintf(stdout, "%s\n", buf);
							outputStatusCallback(buf);
							startedReading = 0;
							if (read == 0) {
								closeItDown();
								exit(1);
							}
						}
						else {
							//fprintf(stdout, "We got (%d channels, %d samplerate, %d samples, %d length\n", audioData.nch, audioData.samplerate, audioData.numsamples, audioData.samplelength);
							memset(metadata, '\000', sizeof(metadata));
							int bufferread = 0;
							if (audioData.metadatalength) {
								bufferread = fread(metadata, 1, audioData.metadatalength, namedPipe);
								if (!strcmp(metadata, "ODDCAST_STOP_ME_NOW")) {
									closeItDown();
									exit(1);
								}
								else {
									setUIMetadata((char *)metadata);
								}
							}
							buffer = (float *)malloc(audioData.samplelength);

							bufferread = fread(buffer, 1, audioData.samplelength, namedPipe);
							if (bufferread != audioData.samplelength) {
								sprintf(buf, "Something terminated the pipe");
								outputStatusCallback(buf);
								startedReading = 0;
								loop = 0;
								if (read == 0) {
									closeItDown();
									exit(1);
								}
							}
							else {
								//fprintf(stdout, "Num Samples = %d\n", audioData.numsamples);
								if (getIsConnected()) {
									int ret = handle_output((float *)buffer, audioData.numsamples, audioData.nch, audioData.samplerate);
								}
								else {
									if (!startedReading) {
										sprintf(buf, "Reading but not broadcasting");
										outputStatusCallback(buf);
										startedReading = 1;
									}
								}
							}
							if (buffer) {
								free(buffer);
							}
						}
				//}
			}
		}
#endif
    return NULL;
}

#ifdef __WXMSW__
enum {
    PU_RESTORE = 10001,
    PU_EXIT,
};

BEGIN_EVENT_TABLE(PopupSystemTrayIcon, wxTaskBarIcon)
    EVT_MENU(PU_RESTORE,    PopupSystemTrayIcon::OnMenuRestore)
    EVT_MENU(PU_EXIT,       PopupSystemTrayIcon::OnMenuExit)
END_EVENT_TABLE()

void PopupSystemTrayIcon::OnMenuRestore(wxCommandEvent& )
{
    frame->Show(TRUE);
}

void PopupSystemTrayIcon::OnMenuExit(wxCommandEvent& )
{
    frame->Close(TRUE);

    // Nudge wxWindows into destroying the dialog, since
    // with a hidden window no messages will get sent to put
    // it into idle processing.
    wxGetApp().ProcessIdle();
}


// Overridables
void PopupSystemTrayIcon::OnMouseMove(wxEvent&)     { }
void PopupSystemTrayIcon::OnLButtonDown(wxEvent&)   { }
void PopupSystemTrayIcon::OnLButtonUp(wxEvent&)     { }
void PopupSystemTrayIcon::OnRButtonDown(wxEvent&)   { }
void PopupSystemTrayIcon::OnRButtonDClick(wxEvent&) { }
void PopupSystemTrayIcon::OnRButtonUp(wxEvent&) { }

void PopupSystemTrayIcon::OnLButtonDClick(wxEvent&)
{
	if (!shown) {
	    frame->Show(TRUE);
		shown = 1;
	}
	else {
	    frame->Show(FALSE);
		shown = 0;
	}
}

#endif

ReconnectTimer::ReconnectTimer() {
	;
}
void ReconnectTimer::Notify() {
	if (!getIsConnected()) {
		if (getReconnectTrigger()) {
			if (reconnectServer()) {
				setReconnectTrigger(0);
			}
		}
	}
}

VUTimer::VUTimer() {
	;
}
void VUTimer::Notify() {
	if (getIsConnected()) {
		if (frame->mainPanel->vu_onoff) { 
			wxRect	vuRect(100, 130, 120, 15);
			frame->mainPanel->Refresh(FALSE, &vuRect);
		}
	}
}



MetatdataUpdateDlg::MetatdataUpdateDlg(wxWindow* parent, int x, int y)
: wxDialog(parent, -1,(const wxChar *) "Update Metadata", wxPoint(x, y), wxSize(-1, -1), wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER,(const wxChar *) "Update Metadata")
{
	wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

	m_metadataLocked = FALSE;
  // 1) top: create wxStaticText with minimum size equal to its default size
  topsizer->Add(
    new wxStaticText( this, -1, _T("Set/Lock Meta Data") ),
    0,                         // make vertically unstretchable
    wxALIGN_CENTER |            // right align text
    wxTOP | wxLEFT | wxRIGHT,  // make border all around except wxBOTTOM
    5 );                      // set border width to 5

	m_MetadataCtrl = new wxTextCtrl(this, -1,(const wxChar *) "",
				wxPoint(-1,-1), wxSize(200, 24), 0,
				wxTextValidator(wxFILTER_NONE, &m_Metadata));


	/*
	wxBoxSizer *box_sizer1 = new wxBoxSizer( wxVERTICAL );
    box_sizer1->Add(new wxStaticText(this, -1, _T("Automatic Metadata")));
    box_sizer1->Add(new wxStaticText(this, -1, _T("Manual Metadata")));
	topsizer->Add(box_sizer1, 1, wxGROW | wxALIGN_LEFT, 10);

	wxBoxSizer *box_sizer2 = new wxBoxSizer( wxVERTICAL );
    box_sizer1->Add(new wxStaticText(this, -1, _T("Automatic Metadata")));
    box_sizer1->Add(new wxStaticText(this, -1, _T("Manual Metadata")));
	topsizer->Add(box_sizer1, 1, wxGROW | wxALIGN_LEFT, 10);
	*/

    // 2.7) And a test of wxGridSizer
    wxFlexGridSizer *gridsizer = new wxFlexGridSizer(2, 5, 5);
    gridsizer->Add(new wxStaticText(this, -1, _T("Metadata")), 0, 0);
    gridsizer->Add(m_MetadataCtrl, 0, wxEXPAND | wxALIGN_LEFT);

	gridsizer->AddGrowableCol(1);

    topsizer->Add(gridsizer, 1, wxGROW | wxALL, 10);

	m_MetadataLockedCtrl = new wxCheckBox(this, -1,(const wxChar *) "Lock Metadata",
				wxPoint(-1,-1), wxSize(-1, -1), 0,
				wxGenericValidator(&m_metadataLocked));

    topsizer->Add(m_MetadataLockedCtrl, 1, wxGROW | wxALL, 10);


  wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
  button_sizer->Add(
     new wxButton( this, wxID_OK,(const wxChar *) "OK" ),
     0,           // make horizontally unstretchable
     wxALL,       // make border all around (implicit top alignment)
     10 );        // set border width to 10
  button_sizer->Add(
     new wxButton( this, wxID_CANCEL,(const wxChar *) "Cancel" ),
     0,           // make horizontally unstretchable
     wxALL,       // make border all around (implicit top alignment)
     10 );        // set border width to 10

  topsizer->Add(
     button_sizer,
     0,                // make vertically unstretchable
     wxALIGN_CENTER ); // no border and centre horizontally


  // don't allow frame to get smaller than what the sizers tell it and also set
  // the initial size as calculated by the sizers
  topsizer->SetSizeHints( this );

  SetSizer( topsizer );
}

MetatdataUpdateDlg::~MetatdataUpdateDlg()
{
	;
}
